<?php

namespace Ecommerce\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\Libs\Util\SMTP;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Utilities 
{
    protected $environment;
    protected $connection;
    protected $fileServerURL;
    protected $smtpUtil;
    
    private $appId;
    private $customSmtpSettings = array();
    
    public function __construct($environment, $connection, $fileServerURL) {
        $this->environment = $environment;
        $this->connection  = $connection;
        $this->fileServerURL = $fileServerURL;
        $this->templatesDir = realpath(__DIR__ . '/Template');
        
        $this->appId = "PORTAL E-COMMERCE";
	}
    
    public function verificaIdSisDos(DTO\Request $request, DTO\Response $response) 
    {
        $params = array(
    		"NRORG" => $this->environment->getNrorg()
    	);  
    	$sql = "SELECT IDSISDOS FROM PARAMGERAL WHERE NRORG = :NRORG";
        
    	$query = $this->connection->prepare($sql);
        
    	$query->execute($params);
    	$resp = $query->fetchAll();
        $response->addDataSet(new DataSet("resp", $resp[0]));
    }
    
    public function getFilialParasupri()
    {
        $query = "SELECT CDFILIAL 
                    FROM PARAMGERALECOMMERCE 
                   WHERE IDFILIALPADRAOSMTP = 'S'";
                    
        $std = $this->connection->prepare($query);
        
        $std->execute();
        
        $parasupri = $std->fetch();
        
        if (empty($parasupri['CDFILIAL'])) {
            throw new \Exception("Central de compras não parametrizada.");
        }
        
        return $parasupri['CDFILIAL'];
    }
    
    public function checkManual(DTO\Request $request, DTO\Response $response) 
    {
        $cdFilCentCmp = $this->getFilialParasupri();
    	
    	if ($cdFilCentCmp != "") {
    	    $manual = $this->getManual($cdFilCentCmp);
    	    
    	    if ($manual != "") {
    	        $path = $this->fileServerURL . $manual;
                
                $response->addDataSet(new DataSet("resp", [$path]));
    	    } else {
    	        $response->addDataSet(new DataSet("resp", [""]));
    	    }
    	} else {
    	    $response->addDataSet(new DataSet("resp", [""]));
    	}
    }
    
    private function getManual($cdFilCentCmp)
    {
        $params = array(
    		"CDFILIAL" => $cdFilCentCmp
    	); 
    	
    	$sql = "SELECT DSMANUALFILESERV FROM PARAMGERALECOMMERCE WHERE CDFILIAL = :CDFILIAL";
    	
    	$query = $this->connection->prepare($sql);
    	
    	$query->execute($params);
	    $manual = $query->fetchAll();
	    
	    return isset($manual[0]) ? $manual[0]["DSMANUALFILESERV"] : "";
    }
    
    public function downloadManual(DTO\Request $request, DTO\Response $response) 
    {
        $row = $request->getDataSet()->getRows()[0];
        
        $path = $row["path"];
        
        $nameArray = explode(".", $path);
        $type = end($nameArray);
        
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        header("Content-Disposition: attachment; filename=Manual." . $type);
        readfile($path);
        exit;
    }
    
    public function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
    
    public function getLogo($idAplicativo)
    {
		$sql = "SELECT URLLOGO 
                  FROM APPCONFIG 
                 WHERE IDAPLICATIVO = :IDAPLICATIVO";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(
            array(
                "IDAPLICATIVO" => $idAplicativo
            )
        );
		$result = $stmt->fetchAll();
		
		if(isset($result[0]) && $result[0]['URLLOGO'] && $result[0]['URLLOGO'] != null && $result[0]['URLLOGO'] != ''){
    		return $this->fileServerURL.$result[0]['URLLOGO'];
        }else{
            return false;
        }
    }
    
    public function getLgpd($cdFilial)
    {
        $sql = "SELECT P.TXAVISOLEGALEMAIL 
                  FROM PARASUPRI P, FILIAL F 
                 WHERE F.CDFILIAL = :P_CDFILIAL 
                   AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(
            array(
                "P_CDFILIAL" => $cdFilial
            )
        );
		$result = $stmt->fetchAll();
        return $result[0]['TXAVISOLEGALEMAIL'];
    }
    
    public function messageSendMail($cdFilial, $assunto, $message, $idAplicativo = 'PORTAL E-COMMERCE', $attrImg = 'width="175px"') 
    {
        $logo = $this->getLogo($idAplicativo);
        $this->getLogoAttributes($logo, $attrImg);
        $lgpd = $this->getLgpd($cdFilial);
        return $this->returnMessageSendMail($logo, $assunto, $message, $attrImg, $lgpd);
    }
    
    public function checkFileExists($url, &$fileContents = "")
    {
	    $url = preg_replace("/^(http:\/\/|https:\/\/)/", "", $url);
    	
    	$curl = curl_init();
    	
    	curl_setopt($curl, CURLOPT_URL, $url);
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
    	curl_setopt($curl, CURLOPT_HEADER, false);
    	curl_exec($curl);
    	
    	$code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    	
    	if ($code == 200) {
    		$status = true;
			$fileContents = curl_exec($curl);
    	} else {
    		$status = false;
    	}
    	
    	curl_close($curl);
    	return $status;
    }

    public function getLogoAttributes(&$logo, &$attrImg)
    {
        if ($logo != false && $this->checkFileExists($logo)) {
            $imgAttributes = getimagesize($logo);
            
            if ($imgAttributes[1] > $imgAttributes[0]) {
                $attrImg = 'height="175px"';
            }
        } else {
            $logo = 'https://www.teknisa.com/imagens/logo-teknisa.png';
        }
         
    }
    
    private function returnMessageSendMail($logo, $assunto, $message, $attrImg, $lgpd)
    {
        $templateEmail = $this->fileGetContentUtf8($this->templatesDir . "/email.html");
        
        $replace = array(
            $attrImg,
            $logo,
            $assunto,
            $message,
            $lgpd
        );
            
        $varReplaced = array(
            '%ATTRIMG%',
            '%LOGOSRC%',
            '%SUBJECT%',
            '%MESSAGE%',
            '%LGPD%'
        );
        
        return str_replace($varReplaced, $replace, $templateEmail);
    }
    
    public function sendMail($subject, $content, $emails)
    {
        $message = $this->messageSendMail($this->unitId, $subject, $content, $this->appId);
        $this->enviaEmail(null, $subject, $emails, $message, $emails);
    }
    
    public function enviaEmail($fromName, $subject, $address, $messageSendMail, $emailDestino, $documentoAnexoFullPath = false, $name = false, $type = false)
    {
        try {
            $this->useCustomSmtp();
            
            if (!empty($fromName)) {
                $this->smtpUtil->setFromName($fromName);
            }
            
            $this->smtpUtil->setSubject($subject);
            
            if($documentoAnexoFullPath && $this->checkFileExists($documentoAnexoFullPath, $fileContents)){
                $this->smtpUtil->clearAttachments();
                $this->smtpUtil->addAttachment($fileContents, $name, 'base64', $type, 'attachment', true);
            }
        
            $this->smtpUtil->setNmAddress($address);
            $this->smtpUtil->sendMail($messageSendMail, $emailDestino);
        } catch (\Exception $e) {
            // Caso ocorra erro com os dados da unidade é feito uma segunda tentativa com mnemonicos
            $smtpMail = InstanceManager::getInstance()->getService('smtpUtil');            
            $smtpMail->setSubject($subject);
            $smtpMail->sendMail($messageSendMail, $emailDestino);
        }
    }
    
    /*********************************************************************************************/
    /**************************************** CUSTOM SMTP ****************************************/
    /*********************************************************************************************/
    
    public function useCustomSmtp()
    {   
        try {
            $this->customSmtpSettings = $this->getCustomSmtpSettings();
            $this->smtpUtil = new SMTP($this->customSmtpSettings);
    
            if (isset($this->customSmtpSettings['fromName'])) {
                $this->smtpUtil->setFromName($this->customSmtpSettings['fromName']);
            }
    
            if (isset($this->customSmtpSettings['from'])) {
                $this->smtpUtil->setNameReplyTo($this->customSmtpSettings['from']);
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getCustomSmtpSettings()
    {
        try {
            $filialParasupri = $this->getFilialParasupri();
            $dadosSMTPFilial = $this->getDadosSMTPFromFilial($filialParasupri);
            
            if(isset($dadosSMTPFilial['DSSMTPEMAILF'])) {
                $settings = self::getSmtpSettingsFromAddress($dadosSMTPFilial['DSSMTPEMAILF']);
                
                if (isset($settings['port'])) {
                    $params['port'] = $settings['port'];
                }
        
                $params['host'] = $settings['host'];
                $params['from'] = $dadosSMTPFilial['DSEMAILFILI'];
                $params['fromName'] = $dadosSMTPFilial['DSEMAILFILI'];
                $params['username'] = $dadosSMTPFilial['DSLOGINSMTP'];
                $params['password'] = $this->getPassword($dadosSMTPFilial['CDSENHEMAILF']);
                
                $securityParameters = $this->getSecurityParameters($dadosSMTPFilial, $params);
                $params = array_merge($params, $securityParameters);
                
                return $params;
            } else {
                throw new \Exception("Dados SMTP não cadastrados");
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    /**
     * Recupera a senha descriptografada. Caso a senha não exista nos parâmetros,
     * retorna uma string vazia.
     * 
     * @param string $password
     * 
     * @return string Contendo a senha devidamente tratada.
     */
    private function getPassword($password)
    {
        if (empty($password)) {
            return '';
        }

        return self::getDelphiPasswordEncrypted($password, false);
    }
    
    /**
     * Recupera informações de segurança para usar na conexão com o SMTP.
     * Utiliza dos parâmetros recuperados para definir as seguintes propriedades:
     * . smtpSecure:
     *      - 'tls', caso o parâmetro IDUTTLSEMAIL esteja marcado;
     *      - 'ssl', caso o parâmetro IDUTSSLEMAIL esteja marcado;
     *      - '', caso ambos os parâmetros acima estejam desmarcados;
     * . smtpAuth:
     *      - true, caso o usuário e senha estejam preenchidos;
     *      - false, caso usuário ou senha estejam em branco;
     * 
     * @param array $dadosSMTPFilial
     * @param array $dadosSMTPCustom
     * 
     * @return array Contendo os parâmetros de segurança.
     */
    private function getSecurityParameters($dadosSMTPFilial, $dadosSMTPCustom)
    {
        $params = [
            'smtpSecure' => '',
            'smtpAuth' => false
        ];
        
        if ($dadosSMTPFilial['IDUTTLSEMAIL'] == 'S') {
            $params['smtpSecure'] = 'tls';
        } else if ($dadosSMTPFilial['IDUTSSLEMAIL'] == 'S') {
            $params['smtpSecure'] = 'ssl';
        }
        
        if (!empty($dadosSMTPCustom['username']) && !empty($dadosSMTPCustom['password'])) {
            $params['smtpAuth'] = true;
        }
        
        return $params;
    }
    
    /**
     * Recupera informações do endereço de SMTP, como:
     * - host;
     * - porta;
     * 
     * @param string $url Url do servidor com ou sem porta.
     * 
     * @return array Contendo as informações encontradas.
     */
    public function getSmtpSettingsFromAddress($url)
    {
        $match = self::splitByCommonDelimiters($url);
        
        if (empty($match)) {
            throw new \Exception('Formato do endereço de SMTP inválido.');
        }
        
        if (count($match) > 1) {
            $params['port'] = $match[1];
        }
        
        $params['host'] = $match[0];
        
        return $params;
    }
        
    public function splitByCommonDelimiters($string)
    {
        $pattern = '/[:;|]+/';
        $cleanedString = self::removeAllWhitespaces($string);
        
        $splittedString = preg_split($pattern, $cleanedString);
        
        return $splittedString;
    }
    
    public function removeAllWhitespaces($string)
    {
        $pattern = '/\s+/';
        
        $cleanedString = preg_replace($pattern, '', $string);
        
        return $cleanedString;
    }

    public function getCentralCompras($codigoFilial)
    {
        $query = "SELECT
                        F.CDFILIAL,
                        F.NMFILIAL,
                        F2.CDFILIAL   AS CDFILCENTCMP,
                        F2.NMFILIAL   AS NMFILCENTCMP,
                        F2.NRORG
                    FROM
                        FILIAL   F,
                        FILIAL   F2
                    WHERE
                        F.CDFILIAL = :CDFILIAL
                        AND F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "CDFILIAL" => $codigoFilial,
        ));
        
        return $std->fetch();
    }

    public function getDadosSMTPFromFilial($filial)
    {
        $query = "SELECT EF.CDSENHEMAILF, EF.DSSMTPEMAILF, NVL(EF.DSLOGINSMTP, EF.DSEMAILFILI) DSLOGINSMTP,
                          NVL(EF.DSEMAILFILI, EF.DSLOGINSMTP) DSEMAILFILI, EF.IDUTTLSEMAIL,
                           EF.IDUTSSLEMAIL, FI.CDFILIAL||' - '||FI.NMFILIAL AS NMFILIAL
                      FROM ENDEFILI EF INNER JOIN FILIAL FI ON EF.CDFILIAL = FI.CDFILIAL
                     WHERE EF.CDFILIAL = :P_CDFILIAL
                       AND EF.IDTPENDEFILI = 'P'";
                       
        $std = $this->connection->prepare($query);
        
        $std->execute(array(
            "P_CDFILIAL" => $filial,
        ));
        
        $smtpFilial = $std->fetch();
        
        if (empty($smtpFilial)) {
            throw new \Exception("Parametrização do servidor SMTP não encontrada.");
        }
        
        return $smtpFilial;
    }

    public function getVisaoSistema()
    {
        $filialLogada = $this->environment->getCdFilial();
        $centralCompras = $this->getCentralCompras($filialLogada)['CDFILCENTCMP'];

        if ($centralCompras == $filialLogada) {
            return 'SUP';
        }

        return 'CMP';
    }
    
    /**
     * Temporário até que uma solução seja implementada no local original desta
     * função.
     * 
     * A original realiza chamada apenas nas funções GetCriptoPass e GetUnCriptoPass,
     * equivalentes ao CriptoPlus e UncriptoPlus do Delphi.
     * 
     * Porém, utilizamos as criptografias convencionais, que até então não eram
     * implementadas la Bib_html.
     * 
     * Foi incluída na 'Bib', rdm 'rdmDBGeral', na issue #238324
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getDelphiPasswordEncrypted($password, $isEncrypt = true, $offset = '2')
    {
        $url = self::getAspUrlParameter() . '/comwebservice/service.asmx?wsdl';
        
        $options = array(
            'encoding'    => 'ISO-8859-1',
            'trace'       => true,
            'exceptions'  => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );
    
        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
            new \SoapVar($offset, XSD_STRING, 'string', "http://www.w3.org/2001/XMLSchema"),
        );
    
        $cryptType = $isEncrypt ? 'GetCripto' : 'GetUncripto';
        
        $params = array(
            'rdm'    => 'Bib.rdmDBGeral',
            'method' => $cryptType,
            'args'   => $args,
        );
    
        try {
            $client = new \SoapClient($url, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
    
        return $encryption;
    }
    
    /**
     * Temporário até que uma solução seja implementada para a função getDelphiPasswordEncrypted.
     * 
     * Teknisa\Libs\Util\Utilities
     */
    public static function getAspUrlParameter() {
        return InstanceManager::getInstance()->getParameter('aspUrl');
    }
    
    /**
     * Transforma os parâmetros de um request do tipo Filter (array de array)
     * em row (array chave/valor), facilitando o acesso ao valor através do nome.
     * Ex.:
     * - Antes:  $array[0]['value'] para recuperar o valor do primeiro parâmetro.
     * - Depois: $array['columnName'] para recuperar o valor do parâmetro 'columnName'.
     * 
     * @param Zeedhi\Framework\DTO\Request\Filter $request
     * 
     * @return array Contendo o nome e valor dos parâmetros encontrados.
     */
    public function getParametersFromCondititions(\Zeedhi\Framework\DTO\Request\Filter $request)
    {
        $conditions = $request->getFilterCriteria()->getConditions();
     
        if (empty($conditions)) return $conditions;
        
        return array_merge(...array_map(function($param) {
        	return [ $param['columnName'] => $param['value'] ];
        }, $conditions));
    }
    
    public function removeSpecialCharacters($string, $removeSpaces = true)
    {
        if ($removeSpaces) {
            $string = str_replace(' ', '-', $string);
        }

        return preg_replace('/[^A-Za-z0-9\ ]/', '', $string);
    }
    
    public function setUnitId($id)
    {
        $this->unitId = $id;
    }
}